@@ -16,15 +16,16 @@
 // under the License.
 
 use super::client::GoogleCloudStorageClient;
+use crate::client::builder::HttpRequestBuilder;
 use crate::client::retry::RetryExt;
 use crate::client::token::TemporaryToken;
 use crate::client::{HttpClient, HttpError, TokenProvider};
 use crate::gcp::{GcpSigningCredentialProvider, STORE};
-use crate::util::{hex_digest, hex_encode, STRICT_ENCODE_SET};
+use crate::util::{STRICT_ENCODE_SET, hex_digest, hex_encode};
 use crate::{RetryConfig, StaticCredentialProvider};
 use async_trait::async_trait;
-use base64::prelude::BASE64_URL_SAFE_NO_PAD;
 use base64::Engine;
+use base64::prelude::BASE64_URL_SAFE_NO_PAD;
 use chrono::{DateTime, Utc};
 use futures::TryFutureExt;
 use http::{HeaderMap, Method};
@@ -88,6 +89,11 @@ pub enum Error {
 
     #[error("Error getting token response body: {}", source)]
     TokenResponseBody { source: HttpError },
+
+    #[error("Error reading pem file: {}", source)]
+    ReadPem {
+        source: rustls_pki_types::pem::Error,
+    },
 }
 
 impl From<Error> for crate::Error {
@@ -123,17 +129,14 @@ pub struct ServiceAccountKey(RsaKeyPair);
 impl ServiceAccountKey {
     /// Parses a pem-encoded RSA key
     pub fn from_pem(encoded: &[u8]) -> Result<Self> {
-        use rustls_pemfile::Item;
-        use std::io::Cursor;
+        use rustls_pki_types::PrivateKeyDer;
+        use rustls_pki_types::pem::PemObject;
 
-        let mut cursor = Cursor::new(encoded);
-        let mut reader = BufReader::new(&mut cursor);
-
-        // Reading from string is infallible
-        match rustls_pemfile::read_one(&mut reader).unwrap() {
-            Some(Item::Pkcs8Key(key)) => Self::from_pkcs8(key.secret_pkcs8_der()),
-            Some(Item::Pkcs1Key(key)) => Self::from_der(key.secret_pkcs1_der()),
-            _ => Err(Error::MissingKey),
+        match PrivateKeyDer::from_pem_slice(encoded) {
+            Ok(PrivateKeyDer::Pkcs8(key)) => Self::from_pkcs8(key.secret_pkcs8_der()),
+            Ok(PrivateKeyDer::Pkcs1(key)) => Self::from_der(key.secret_pkcs1_der()),
+            Ok(_) => Err(Error::MissingKey),
+            Err(source) => Err(Error::ReadPem { source }),
         }
     }
 
@@ -222,6 +225,7 @@ struct TokenClaims<'a> {
 struct TokenResponse {
     access_token: String,
     expires_in: u64,
+    id_token: Option<String>,
 }
 
 /// Self-signed JWT (JSON Web Token).
@@ -424,7 +428,7 @@ impl TokenProvider for InstanceCredentialProvider {
     /// Since the connection is local we need to enable http access and don't actually use the client object passed in.
     /// Respects the `GCE_METADATA_HOST`, `GCE_METADATA_ROOT`, and `GCE_METADATA_IP`
     /// environment variables.
-    ///  
+    ///
     /// References: <https://googleapis.dev/python/google-auth/latest/reference/google.auth.environment_vars.html>
     async fn fetch_token(
         &self,
@@ -494,7 +498,7 @@ impl TokenProvider for InstanceSigningCredentialProvider {
     /// Since the connection is local we need to enable http access and don't actually use the client object passed in.
     /// Respects the `GCE_METADATA_HOST`, `GCE_METADATA_ROOT`, and `GCE_METADATA_IP`
     /// environment variables.
-    ///  
+    ///
     /// References: <https://googleapis.dev/python/google-auth/latest/reference/google.auth.environment_vars.html>
     async fn fetch_token(
         &self,
@@ -603,6 +607,37 @@ struct EmailResponse {
     email: String,
 }
 
+#[derive(Debug, Deserialize)]
+struct IdTokenClaims {
+    email: String,
+}
+
+async fn get_token_response(
+    client_id: &str,
+    client_secret: &str,
+    refresh_token: &str,
+    client: &HttpClient,
+    retry: &RetryConfig,
+) -> Result<TokenResponse> {
+    client
+        .post(DEFAULT_TOKEN_GCP_URI)
+        .form([
+            ("grant_type", "refresh_token"),
+            ("client_id", client_id),
+            ("client_secret", client_secret),
+            ("refresh_token", refresh_token),
+        ])
+        .retryable(retry)
+        .idempotent(true)
+        .send()
+        .await
+        .map_err(|source| Error::TokenRequest { source })?
+        .into_body()
+        .json::<TokenResponse>()
+        .await
+        .map_err(|source| Error::TokenResponseBody { source })
+}
+
 impl AuthorizedUserSigningCredentials {
     pub(crate) fn from(credential: AuthorizedUserCredentials) -> crate::Result<Self> {
         Ok(Self { credential })
@@ -613,16 +648,42 @@ impl AuthorizedUserSigningCredentials {
         client: &HttpClient,
         retry: &RetryConfig,
     ) -> crate::Result<String> {
+        let response = get_token_response(
+            &self.credential.client_id,
+            &self.credential.client_secret,
+            &self.credential.refresh_token,
+            client,
+            retry,
+        )
+        .await?;
+
+        // Extract email from id_token if available
+        if let Some(id_token) = response.id_token {
+            // Split the JWT string by dots to get the payload section
+            let parts: Vec<&str> = id_token.split('.').collect();
+            if parts.len() == 3 {
+                // Decode the base64-encoded payload (middle part)
+                if let Ok(payload) = BASE64_URL_SAFE_NO_PAD.decode(parts[1]) {
+                    // Parse the payload as JSON and extract the email
+                    if let Ok(claims) = serde_json::from_slice::<IdTokenClaims>(&payload) {
+                        return Ok(claims.email);
+                    }
+                }
+                // If any of the parsing steps fail, fallback to other method
+            }
+        }
+
+        // Fallback to the original method if id_token is not available or invalid
         let response = client
             .get("https://oauth2.googleapis.com/tokeninfo")
-            .query(&[("access_token", &self.credential.refresh_token)])
+            .query(&[("access_token", response.access_token)])
             .send_retry(retry)
             .await
             .map_err(|source| Error::TokenRequest { source })?
             .into_body()
             .json::<EmailResponse>()
             .await
-            .map_err(|source| Error::TokenResponseBody { source })?;
+            .map_err(|source: HttpError| Error::TokenResponseBody { source })?;
 
         Ok(response.email)
     }
@@ -638,7 +699,6 @@ impl TokenProvider for AuthorizedUserSigningCredentials {
         retry: &RetryConfig,
     ) -> crate::Result<TemporaryToken<Arc<GcpSigningCredential>>> {
         let email = self.client_email(client, retry).await?;
-
         Ok(TemporaryToken {
             token: Arc::new(GcpSigningCredential {
                 email,
@@ -658,23 +718,14 @@ impl TokenProvider for AuthorizedUserCredentials {
         client: &HttpClient,
         retry: &RetryConfig,
     ) -> crate::Result<TemporaryToken<Arc<GcpCredential>>> {
-        let response = client
-            .post(DEFAULT_TOKEN_GCP_URI)
-            .form([
-                ("grant_type", "refresh_token"),
-                ("client_id", &self.client_id),
-                ("client_secret", &self.client_secret),
-                ("refresh_token", &self.refresh_token),
-            ])
-            .retryable(retry)
-            .idempotent(true)
-            .send()
-            .await
-            .map_err(|source| Error::TokenRequest { source })?
-            .into_body()
-            .json::<TokenResponse>()
-            .await
-            .map_err(|source| Error::TokenResponseBody { source })?;
+        let response = get_token_response(
+            &self.client_id,
+            &self.client_secret,
+            &self.refresh_token,
+            client,
+            retry,
+        )
+        .await?;
 
         Ok(TemporaryToken {
             token: Arc::new(GcpCredential {
@@ -720,7 +771,7 @@ impl GCSAuthorizer {
         let email = &self.credential.email;
         let date = self.date.unwrap_or_else(Utc::now);
         let scope = self.scope(date);
-        let credential_with_scope = format!("{}/{}", email, scope);
+        let credential_with_scope = format!("{email}/{scope}");
 
         let mut headers = HeaderMap::new();
         headers.insert("host", DEFAULT_GCS_SIGN_BLOB_HOST.parse().unwrap());
@@ -772,8 +823,7 @@ impl GCSAuthorizer {
         let (canonical_headers, signed_headers) = Self::canonicalize_headers(headers);
 
         format!(
-            "{}\n{}\n{}\n{}\n\n{}\n{}",
-            verb, path, query, canonical_headers, signed_headers, DEFAULT_GCS_PLAYLOAD_STRING
+            "{verb}\n{path}\n{query}\n{canonical_headers}\n\n{signed_headers}\n{DEFAULT_GCS_PLAYLOAD_STRING}"
         )
     }
 
@@ -854,6 +904,26 @@ impl GCSAuthorizer {
     }
 }
 
+pub(crate) trait CredentialExt {
+    /// Apply bearer authentication to the request if the credential is not None
+    fn with_bearer_auth(self, credential: Option<&GcpCredential>) -> Self;
+}
+
+impl CredentialExt for HttpRequestBuilder {
+    fn with_bearer_auth(self, credential: Option<&GcpCredential>) -> Self {
+        match credential {
+            Some(credential) => {
+                if credential.bearer.is_empty() {
+                    self
+                } else {
+                    self.bearer_auth(&credential.bearer)
+                }
+            }
+            None => self,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
